if x is "abc":
    pass

if 123 is not y:
    pass

if 123 is \
        not y:
    pass

if "123" is x < 3:
    pass

if "123" != x is 3:
    pass

if ("123" != x) is 3:
    pass

if "123" != (x is 3):
    pass
