import a
# Don't take this comment into account when determining whether the next import can fit on one line.
from b import c
from d import e  # Do take this comment into account when determining whether the next import can fit on one line.
