x = "𝐁ad string"
y = "−"


def f():
    """Here's a docstring with an unusual parenthesis: ）"""
    # And here's a comment with an unusual punctuation mark: ᜵
    ...


def g():
    """Here's a docstring with a greek rho: ρ"""
    # And here's a comment with a greek alpha: ∗
    ...
