# Comment 1
# Comment 2
import D

# Comment 3a
import C

# Comment 3b
import C

import B  # Comment 4

# Comment 5

# Comment 6
from A import (
    a,  # Comment 7
    b,
    c,  # Comment 8
)
from A import (
    a,  # Comment 9
    b,  # Comment 10
    c,  # Comment 11
)
