import a
import b
x = 1
import os
import sys
def f():
    pass
if True:
    x = 1
    import collections
    import typing
class X: pass
y = 1
import os
import sys
"""Docstring"""

if True:
    import os


def f():
    pass


if True:
    import os
def f():
    pass

if True:
    x = 1
    import collections
    import typing
    class X: pass

if True:
    x = 1
    import collections
    import typing
    def f(): pass
