def f():
    "Here's a line without a period"
    ...


def f():
    """Here's a line without a period"""
    ...


def f():
    """
    Here's a line without a period,
    but here's the next line
    """
    ...


def f():
    """Here's a line without a period"""
    ...


def f():
    """
    Here's a line without a period,
    but here's the next line"""
    ...


def f():
    """
    Here's a line without a period,
    but here's the next line with trailing space """
    ...



def f():
    r"Here's a line without a period"
    ...


def f():
    r"""Here's a line without a period"""
    ...


def f():
    r"""
    Here's a line without a period,
    but here's the next line
    """
    ...


def f():
    r"""Here's a line without a period"""
    ...


def f():
    r"""
    Here's a line without a period,
    but here's the next line"""
    ...


def f():
    r"""
    Here's a line without a period,
    but here's the next line with trailing space """
    ...
