#: E731
f = lambda x: 2 * x
#: E731
f = lambda x: 2 * x
#: E731
while False:
    this = lambda y, z: 2 * x


f = object()
f.method = lambda: "Method"

f = {}
f["a"] = lambda x: x ** 2

f = []
f.append(lambda x: x ** 2)

f = g = lambda x: x ** 2

lambda: "no-op"
