for i in range(10):
    continue
else:
    continue

i = 0
while i < 10:
    i += 1
    continue


def f():
    for i in range(10):
        continue

    continue


class Foo:
    continue


continue
