from . import sibling
from .sibling import example

from .. import parent
from ..parent import example

from ... import grandparent
from ...grandparent import example

import other
import other.example
from other import example
